package com.zhili.stationcontrol.chargeserver.component;

import com.zhili.stationcontrol.chargeserver.handler.ChargeMessageDecoder;
import com.zhili.stationcontrol.chargeserver.handler.ChargeMessageEncoder;
import com.zhili.stationcontrol.chargeserver.handler.ChargeMessageHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.LineBasedFrameDecoder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Component;

/**
 * @author ：HuangBin
 * @description：TODO
 * @date ：2022/10/24 10:36
 */
@Slf4j
@Component
@EnableAsync
//192.168.1.200:9988
//255.255.0.0
//192.168.1.1
public class ChargeServer {
    @Async
    public void run() {
        EventLoopGroup bossGroup = new NioEventLoopGroup(1);
        EventLoopGroup workerGroup = new NioEventLoopGroup();
        try {
            ServerBootstrap server = new ServerBootstrap();
            server.group(bossGroup, workerGroup)
                    .channel(NioServerSocketChannel.class)
                    .childOption(ChannelOption.SO_KEEPALIVE, true)
                    .childOption(ChannelOption.TCP_NODELAY, true)
                    .option(ChannelOption.SO_BACKLOG, 1024)
                    .childHandler(new ChannelInitializer<SocketChannel>() {
                        @Override
                        protected void initChannel(SocketChannel socketChannel) throws Exception {
                            socketChannel.pipeline()
                                    .addLast(new ChargeMessageDecoder())
                                    .addLast(new ChargeMessageEncoder())
                                    .addLast(new ChargeMessageHandler());
                        }
                    });
            ChannelFuture channelFuture = server.bind(9988).sync();
            channelFuture.channel().closeFuture().sync();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            bossGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
        }
    }
}
