package com.zhili.stationcontrol.service.impl;

import com.zhili.stationcontrol.chargeserver.dto.ChargeMessage;
import com.zhili.stationcontrol.chargeserver.dto.ChargeMessage.CommandType;
import com.zhili.stationcontrol.chargeserver.dto.ChargeRate;
import com.zhili.stationcontrol.chargeserver.util.ChargeCenter;
import com.zhili.stationcontrol.service.ChargeService;
import com.zhili.stationcontrol.util.BytesUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ArrayUtils;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Random;

/**
 * @author ：HuangBin
 * @description：TODO
 * @date ：2022/10/27 13:40
 */
@Service
@Slf4j
public class ChargeServiceSHImpl implements ChargeService {
    static Random r = new Random();
    static byte version = (byte) 0x02;

    @Override
    public int startImmediatelyUtilFull(String deviceNo, int gunNo, int stopPwd, String userId, String chrgId) {
        ChargeMessage m = new ChargeMessage();
        m.setVersion(version);
        m.setSerialNo(randomSerialNo());
        m.setCommand(CommandType.CHARGE_COMMAND);
        byte[] data = new byte[100];
        //设置枪号
        data[4] = (byte) gunNo;
        //设置即时充电
        BytesUtil.setSubBytes(data, 5, BytesUtil.fromIntWithLowerFirst(0, 4));
        //停止密码
        BytesUtil.setSubBytes(data, 9, BytesUtil.fromIntWithLowerFirst(stopPwd, 4));
        //充电策略(0,充满为止)
        BytesUtil.setSubBytes(data, 13, BytesUtil.fromIntWithLowerFirst(0, 4));
        //用户卡号
        byte[] userIdBytes = BytesUtil.fromString(userId, 32, (byte) 0);
        BytesUtil.setSubBytes(data, 30, userIdBytes);
        //充电流水号
        byte[] chrgIdBytes = BytesUtil.fromString(chrgId, 32, (byte) 0);
        BytesUtil.setSubBytes(data, 67, chrgIdBytes);
        m.setData(data);
        ChargeCenter.send("signIn", deviceNo, m);
        return 0;
    }

    @Override
    public int stopImmediately(String deviceNo, int gunNo) {
        ChargeMessage m = new ChargeMessage();
        m.setVersion(version);
        m.setSerialNo(randomSerialNo());
        m.setCommand(CommandType.CONTROL_COMMAND);
        byte[] data = new byte[16];
        //设置枪号
        data[4] = (byte) gunNo;
        //设置起始命令地址，停止充电为2
        BytesUtil.setSubBytes(data, 5, BytesUtil.fromIntWithLowerFirst(2, 4));
        //设置命令个数
        data[9] = (byte) 1;
        //设置命令参数长度
        BytesUtil.setSubBytes(data, 10, BytesUtil.fromIntWithLowerFirst(4, 2));
        //设置命令参数
        BytesUtil.setSubBytes(data, 12, BytesUtil.fromIntWithLowerFirst(0x55, 4));
        m.setData(data);
        ChargeCenter.send("signIn", deviceNo, m);
        return 0;
    }

    @Override
    public int queryChargeHistory(String deviceNo, int backIndex) {
        ChargeMessage m = new ChargeMessage();
        m.setVersion(version);
        m.setSerialNo(randomSerialNo());
        m.setCommand(CommandType.QUERY_CHARGE_COMMAND);
        byte[] data = new byte[8];
        BytesUtil.setSubBytes(data, 4, BytesUtil.fromIntWithLowerFirst(backIndex, 4));
        m.setData(data);
        ChargeCenter.send("signIn", deviceNo, m);
        return 0;
    }

    @Override
    public int setChargeRate(String deviceNo, List<ChargeRate> rates) {
        ChargeMessage m = new ChargeMessage();
        m.setVersion(version);
        m.setSerialNo(randomSerialNo());
        m.setCommand(CommandType.SET_RATE_COMMAND);
        byte[] data = new byte[8 * 12];
        for (int i = 0; i < rates.size(); i++) {
            ChargeRate chargeRate = rates.get(i);
            data[8 * i] = (byte) chargeRate.getStartHour();
            data[8 * i + 1] = (byte) chargeRate.getStartMinute();
            data[8 * i + 2] = (byte) chargeRate.getEndHour();
            data[8 * i + 3] = (byte) chargeRate.getEndMinute();
            log.info("----" + Math.round(chargeRate.getRate() * 100));
            BytesUtil.setSubBytes(data, 8 * i + 4, BytesUtil.fromIntWithLowerFirst(Math.round(chargeRate.getRate() * 100), 4));
        }

        m.setData(data);
        ChargeCenter.send("signIn", deviceNo, m);
        return 0;
    }

    static byte randomSerialNo() {
        return BytesUtil.fromIntWithLowerFirst(r.nextInt(255), 1)[0];
    }
}
